/*
 * $Id: ArtifactUtils.java,v 1.9.2.1 2002/09/17 01:52:20 jsergent Exp $
 * ArtifactUtils.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): Ana Sanz Merino [University of California Berkeley, 2003]
 */

package com.sun.liberty;

import ucb.eecs.sahara.sso.DebugUtils;

import com.sun.liberty.saml.SAMLPRequest;
import com.sun.liberty.saml.SAMLPResponse;
import com.sun.liberty.saml.SAMLPSoapRequest;
import com.sun.liberty.saml.SAMLPSoapResponse;
import java.security.SecureRandom;
import java.util.List;
import java.util.LinkedList;
import javax.xml.soap.SOAPMessage;

/**
 * ArtifactUtils provides utility functions for implementing the
 * artifact-based profiles of the single sign-on protocol.
 * 
 * @author	Jonathan Sergent
 * @version	$Revision: 1.9.2.1 $
 */

public class ArtifactUtils {
    /** Common instance of ProviderDescriptorFactory */
    private static ProviderDescriptorFactory dir;
    /** Common instance of the ProviderDescriptor of this provider */
    private static ProviderDescriptor local;
    /** Common random number generator */
    private static SecureRandom random = new SecureRandom();

    static {
	try {
	    dir = ProviderDescriptorFactory.getInstance();
	    local = dir.findLocal();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    /**
     * Retrieves the message corresponding with the given artifact via
     * SOAP.  Assumes the first 20 bytes of the artifact are the succinct
     * provider ID of the provider to whom the SOAP message should be sent
     * to look up the message, as specified by the Liberty bindings and
     * profiles spec.
     *
     * @param artifactStr The artifact, encoded using base64.
     * @return The samlp:Response received in response to the request
     * for the artifact.
     */
    public static SAMLPResponse dereferenceArtifact(String artifactStr) throws Exception {
        DebugUtils.debugTime("ArtifactUtils", "Entering dereferenceArtifact",
                             DebugUtils.LOW_LEVEL_FUNC);

	byte[] artifactBits = URLUtils.base64Decode(artifactStr);
	byte[] succinctID = new byte[20];
	for (int i = 0; i < 20; i++) {
	    succinctID[i] = artifactBits[2+i];
	}
        DebugUtils.debugTime("ArtifactUtils", "base64 decoded",
                             DebugUtils.LOW_LEVEL_FUNC);

	ProviderDescriptor provider = dir.find(succinctID);
	if (Config.debug) { System.err.println("provider:" + provider); }
	SAMLPSoapRequest request = new SAMLPSoapRequest();
	request.setAssertionArtifact(artifactStr);
	List s = new LinkedList();
	s.add("lib:AuthenticationStatementType");
	request.setRespondWith(s);
	request.setSignedBy(local.getProviderID());
	SOAPMessage soapResponse;
	if (Config.debug) {
	    System.err.println("provider:");
	    System.err.println(provider.getProviderID());
	    System.err.println(provider.getSoapEndpoint());
	}

        DebugUtils.debugTime("ArtifactUtils", "Invoking send SOAP message",
                             DebugUtils.LOW_LEVEL_FUNC);
	soapResponse = SoapUtils.send(request,
				      provider.getSoapEndpoint().toString());
        DebugUtils.debugTime("ArtifactUtils", "SOAP response received",
                             DebugUtils.LOW_LEVEL_FUNC);

	if (Config.debug) {
	    System.err.println("dereferenceArtifact: got reply:");
	    soapResponse.writeTo(System.err);
	}
	SAMLPSoapResponse response = new SAMLPSoapResponse();
	response.setReceivedFrom(provider.getProviderID());
	response.parse(soapResponse);
        DebugUtils.debugTime("ArtifactUtils", "SOAP response parsed",
                             DebugUtils.LOW_LEVEL_FUNC);
	return response;
    }

    /**
     * Generates an artifact for the message, registers it, and returns
     * the artifact.
     *
     * @param audience The provider ID of the intended audience of the
     * artifact (only this provider will be able to dereference it via SOAP).
     * @param msg The message.
     * @return The artifact.
     */
    public static byte[] getArtifactFor(String audience, LibertyArtifactMessage msg) {
	ProviderDescriptor l = dir.findLocal();
	byte[] artifact = generateArtifact(l.getProviderID());
	ArtifactManager.registerArtifact(artifact, msg, audience);
	return artifact;
    }

    /**
     * Generates a new artifact with the given provider ID.  The succinct
     * version of the provider ID is embeded into the artifact.  The
     * recipient will contact this provider to dereference the artifact.
     *
     * @param id The provider ID of this provider
     */
    public static byte[] generateArtifact(String id) {
	ProviderDescriptor prov = dir.find(id);
	byte[] sid = prov.getProviderSuccinctID();
	byte[] artifact = new byte[42];
	artifact[0] = 0x00;
	artifact[1] = 0x03;
	for (int i = 0; i < 20; i++) {
	    artifact[2+i] = sid[i];
	}
	byte[] handle = new byte[20];
	random.nextBytes(handle);
	for (int i = 0; i < 20; i++) {
	    artifact[22+i] = handle[i];
	}
	return artifact;
    }

    /**
     * Constructs a URL query string to be used to pass an artifact to
     * a service provider.
     *
     * @param audience The intended audience of the message
     * @param msg The message
     * @param relay The value of the RelayState parameter to be passed
     * @return The URL query string (no leading '?')
     */
    public static String getArtifactQueryFor(String audience, LibertyArtifactMessage msg, String relay) throws Exception {
	byte[] artifact = msg.getArtifactFor(audience);
	StringBuffer query = new StringBuffer();
	URLUtils.addQueryParam(query, "SAMLart",
			       URLUtils.base64Encode(artifact));
	if (relay != null) {
	    URLUtils.addQueryParam(query, "LRURL", relay);
	}
	return query.toString();
    }
}
