/*
 * $Id: FederationTerminationNotificationProcessor.java,v 1.4.2.1 2002/09/17 01:52:21 jsergent Exp $
 * FederationTerminationNotificationProcessor.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty;
import com.sun.liberty.saml.SAMLNameIdentifier;

/**
 * Processes FederationTerminationNotification messages.
 * 
 * @author Jonathan Sergent
 * @version $Revision: 1.4.2.1 $
 */

public class FederationTerminationNotificationProcessor implements RequestProcessor {
    /** Common instance of provider metadata */
    private static ProviderDescriptorFactory dir;

    static {
	try {
	    dir = ProviderDescriptorFactory.getInstance();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    /**
     * Processes the FederationTerminationNotification.
     *
     * @param	session	The current session.  Null if this is a back
     *			channel SOAP request.
     *
     * Note:
     * Since we may host both an IDP and an SP, it's not necessarily clear
     * whether the incoming request is for the IDP or the SP.  (Theoretically
     * we could use different URLs to differentiate.)  To accomodate this,
     * we check to see whether the requesting party is an SP or an IDP and
     * we terminate federation accordingly.  If it is both an IDP and an SP,
     * we terminate both federations, but that probably won't happen.
     */
    public RequestProcessorAction process(
	    LibertyFederationTerminationNotification request,
	    LibertyUserSession session) throws Exception
    {
	String signedBy = request.getSignedBy();
	if (signedBy == null) {
	    if (Config.debug) { System.err.println("FedTerm isn't signed"); }
	    return RequestProcessorAction.FAILED;
	}
	String providerID = request.getProviderID();
	if (providerID == null) {
	    if (Config.debug) { System.err.println("FedTerm missing prov ID"); }
	    return RequestProcessorAction.FAILED;
	}
	if (!providerID.equals(signedBy)) {
	    if (Config.debug) { System.err.println("FedTerm signed wrong"); }
	    return RequestProcessorAction.FAILED;
	}
	ProviderDescriptor prov = dir.find(providerID);
	if (prov == null) {
	    /* signature check should have failed, but... */
	    if (Config.debug) { System.err.println("Can't find provider?"); }
	    return RequestProcessorAction.FAILED;
	}

	LibertyUser user = (session == null) ? null : session.getUser();

	SAMLNameIdentifier nameID = request.getNameIdentifier();

	LibertyUser requestedUser = null;

	if (prov instanceof SPDescriptor) {
	    requestedUser = LibertyUser.find((SPDescriptor) prov, nameID);
	} 

	if (requestedUser == null && prov instanceof IDPDescriptor) {
	    requestedUser = LibertyUser.find((IDPDescriptor) prov, nameID);
	}

	if (requestedUser == null) {
	    /* Federation was probably already terminated */
	    if (Config.debug) { System.err.println("Can't find user"); }
	    return RequestProcessorAction.FAILED;
	}

	if (user == null) {
	    if (session != null) {
		return RequestProcessorAction.ATTEMPT_USER_AUTH;
	    } else {
		/* Back channel request -- sig is enough to authenticate. */
		user = requestedUser;
	    }
	} else {
	    /*
	     * Front channel request -- make sure we are asking to
	     * terminate the named user.  This will fail if the FC
	     * request is sent but the user is not logged in yet (since
	     * the user did a local login at the SP, for example) and
	     * the user logs in as the *wrong* IDP identity.
	     */
	    if (!requestedUser.getUsername().equals(user.getUsername())) {
		System.err.println("WARNING: Attempt to terminate federation"
				   + " for another user");
		return RequestProcessorAction.FAILED;
	    }
	}

	try {
	    SPDescriptor sp = (SPDescriptor) prov;
	    user.setFederationStatus(sp, false);
	} catch (ClassCastException e) {
	    if (Config.debug) { System.err.println("Isn't an SP..."); }
	}

	try {
	    IDPDescriptor idp = (IDPDescriptor) prov;
	    user.setFederationStatus(idp, false);
	} catch (ClassCastException e) {
	    if (Config.debug) { System.err.println("Isn't an IDP..."); }
	}

	return RequestProcessorAction.RETURN_RESPONSE;
    }
}
