/*
 * $Id: LibertyURLAuthnRequest.java,v 1.9.2.1 2002/09/17 01:52:24 jsergent Exp $
 * LibertyURLAuthnRequest.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty;

import java.net.URLDecoder;
import java.net.URLEncoder;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.security.SignatureException;
import java.util.Iterator;
import java.util.List;
import java.util.LinkedList;
import java.util.StringTokenizer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Subclass of LibertyAuthnRequest that implements the profiles based on
 * URL query strings (all of those except for the LECP profile).
 * @author Jonathan Sergent
 * @version $Revision: 1.9.2.1 $
 */
public class LibertyURLAuthnRequest extends LibertyAuthnRequest implements LibertyURLMessage {
    /**
     * Formats the message into a URL query string.
     *
     * @return  String suitable for appending to a URL or including in a
     *          POST body.
     *          Does not including a leading "?".
     */
    public String getQuery() throws Exception {
	StringBuffer u = new StringBuffer();
	URLUtils.addQueryParam(u, "RequestID", getRequestID());
	URLUtils.addQueryParam(u, "MajorVersion", getMajorVersion());
	URLUtils.addQueryParam(u, "MinorVersion", getMinorVersion());
	URLUtils.addQueryParam(u, "IssueInstant",
			formatDate(getIssueInstant()));
	URLUtils.addQueryParam(u, "ProviderID", getProviderID());
	URLUtils.addQueryParam(u, "ForceAuthn", getForceAuthn());
	URLUtils.addQueryParam(u, "IsPassive", getIsPassive());
	URLUtils.addQueryParam(u, "Federate", getFederate());
	URLUtils.addQueryParam(u, "ProtocolProfile", getProtocolProfile().toString());
	LibertyRequestedAuthnContext ac = getAuthnContext();
	if (ac != null) {
	    URLUtils.addQueryParam(u, "AuthnContextMinimumClassRef",
			    ac.getMinimumClassRef());
	    URLUtils.addQueryParam(u, "AuthnContextStatementRef",
			    ac.getStatementRefs());
	    URLUtils.addQueryParam(u, "AuthenticationContextClassRef",
			    ac.getClassRefs());
	}
	URLUtils.addQueryParam(u, "RelayState", getRelayState());

	if (getSignedBy() != null) {
	    /*
	     * FIXME: We don't use the signedBy value, we just get the first
	     * local provider descriptor.  If we honor the signedBy, then we
	     * can deal with multiple local provider descriptors and pick the
	     * right one for signature.
	     */
	    URLUtils.sign(u);
	}
	return u.toString();
    }

    public void parse(String query) throws Exception {
	Iterator i = URLUtils.tokenize(query);
	StringBuffer soFar = null;
	String sigalg = null;
	
	while (i.hasNext() && getSignedBy() == null) {
	    java.util.Map.Entry e = (java.util.Map.Entry) i.next();
	    String rawkey = (String) e.getKey();
	    String rawvalue = (String) e.getValue();
	    String key = URLDecoder.decode(rawkey);
	    String value = URLDecoder.decode(rawvalue);
	    if (key.equals("RequestID")) {
		setRequestID(value);
	    } else if (key.equals("MajorVersion")) {
		setMajorVersion(Integer.valueOf(value).intValue());
	    } else if (key.equals("MinorVersion")) {
		setMinorVersion(Integer.valueOf(value).intValue());
	    } else if (key.equals("IssueInstant")) {
		setIssueInstant(parseDate(value));
	    } else if (key.equals("ProviderID")) {
		setProviderID(value);
	    } else if (key.equals("ForceAuthn")) {
		setForceAuthn(value.equals("1") || value.equals("true"));
	    } else if (key.equals("IsPassive")) {
		setIsPassive(value.equals("1") || value.equals("true"));
	    } else if (key.equals("Federate")) {
		setFederate(value.equals("1") || value.equals("true"));
	    } else if (key.equals("ProtocolProfile")) {
		setProtocolProfile(LibertyProfile.valueOf(value));
	    } else if (key.equals("AuthnContextMinimumClassRef")) {
		LibertyRequestedAuthnContext ac = new LibertyRequestedAuthnContext();
		ac.setMinimumClassRef(value);
		setAuthnContext(ac);
	    } else if (key.equals("AuthnContextStatementRef")) {
		LibertyRequestedAuthnContext ac = new LibertyRequestedAuthnContext();
		StringTokenizer stmttok = new StringTokenizer(value, " ", true);
		List stmtRefs = new LinkedList();
		while (stmttok.hasMoreTokens()) {
		    stmtRefs.add(stmttok.nextToken());
		}
		ac.setStatementRefs(stmtRefs);
		setAuthnContext(ac);
	    } else if (key.equals("AuthenticationContextClassRef")) {
		LibertyRequestedAuthnContext ac = new LibertyRequestedAuthnContext();
		StringTokenizer classtok = new StringTokenizer(value, " ", true);
		List classRefs = new LinkedList();
		while (classtok.hasMoreTokens()) {
		    classRefs.add(classtok.nextToken());
		}
		ac.setClassRefs(classRefs);
		setAuthnContext(ac);
	    } else if (key.equals("RelayState")) {
		setRelayState(value);
	    } else if (key.equals("SigAlg")) {
		sigalg = value;
	    } else if (key.equals("Signature")) {
		if (URLUtils.verify(soFar, value, sigalg, getProviderID())) {
		    setSignedBy(getProviderID());
		} else {
		    throw new Exception("Signature validation failed");
		}
		break;
	    }
	    /* else skip unknown key */

	    if (soFar == null) {
		soFar = new StringBuffer();
	    } else {
		soFar.append("&");
	    }
	    soFar.append(rawkey);
	    soFar.append("=");
	    soFar.append(rawvalue);	    
	}

	// If t.hasMoreTokens() then we ignore them since they are unsigned
    }

    /**
     * Formats the message into an HTML fragment which will cause the
     * user agent to POST the proper URL query string.
     *
     * @return  An HTML fragment (a <FORM> tag and the contents thereof)
     */
    public String getHTMLFragment(String url) throws Exception {
	return URLUtils.doHTMLConversion(getQuery(), url);
    }
}
