/*
 * $Id: LibertyUserSession.java,v 1.12.2.1 2002/09/17 01:52:25 jsergent Exp $
 * LibertyUserSession.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): Ana Sanz Merino [University of California Berkeley, 2003]
 */

package com.sun.liberty;
import com.sun.liberty.saml.SAMLNameIdentifier;
import java.security.SecureRandom;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.HashSet;
import javax.security.auth.Subject;

/**
 * Interface which allows the common request processing code to obtain
 * information about the principal if known.  Needs to be implemented
 * differently depending on the particular user management and identification
 * implementation at this provider.
 * XXX: Should use SAMLNameIdentifier, not Name.
 * 
 * @author Jonathan Sergent
 * @version $Revision: 1.12.2.1 $
 */
public class LibertyUserSession {
    private LibertyUser user = null;
    private String authnProfile;
    private Set spList = null;
    private boolean pendingSLO = false;
    private List pendingSLOImages = new LinkedList();
    private List pendingSLORedirects = new LinkedList();
    private boolean pendingSLOWebBugs = false;
    private LibertyAuthnRequest pendingAuthnRequest = null;
    private String pendingRelayURL = null;
    private String pendingIDP = null;
    private String fcNonce = null;
    private LibertyAssertion authnAssertion = null;
    private static SecureRandom random = new SecureRandom();

    /*
     * Static methods used to get access to user session information from the
     * back channel.
     */
    /**
     * Finds the user's session based on a name provided by an IDP.
     * @param idp the IDP.
     * @param idpName the name provided by the IDP.
     */
    public static LibertyUserSession findSession(IDPDescriptor idp,
				SAMLNameIdentifier idpName) throws Exception {
        LibertyUser u = LibertyUser.find(idp, idpName);
        return doFindSession(u);
    }

    /**
     * Finds the user's session based on a name provided by an SP.
     * @param sp the SP.
     * @param spName the name provided by the SP.
     */
    public static LibertyUserSession findSession(SPDescriptor sp,
				SAMLNameIdentifier spName) throws Exception {
        LibertyUser u = LibertyUser.find(sp, spName);
        return doFindSession(u);
    }

    private static LibertyUserSession doFindSession(LibertyUser u) 
                                                            throws Exception {
        LibertyUserSession s = (LibertyUserSession) u.getCurrentSession();
        if (s == null) {
            s = new LibertyUserSession();
            s.setUser(u);
            u.setCurrentSession(s);
        }
        return s;
    }

    /**
     * Gets a reference to the user data object for the user authenticated in
     * this session.
     *
     * @return	A LibertyUser object for the user authenticated in this session.
     * 		Returns null if the user hasn't been identified yet.
     */
    public LibertyUser getUser() {
        return user;
    }

    /**
     * Sets the user reference based on information from a remote identity 
     * provider.
     *
     * @param idp the IDP.
     * @param idpProvidedname the name provided by the IDP
     */
    public void setUser(IDPDescriptor idp,
			SAMLNameIdentifier idpProvidedName) throws Exception {
	if (user != null && user.getCurrentSession() == this) {
	    user.setCurrentSession(null);
	}
        setUser(LibertyUser.find(idp, idpProvidedName));
        user.setCurrentSession(this);
    }

    /**
     * Sets the user reference to point to the given user.
     *
     * @param user the user.
     */
    public void setUser(LibertyUser user) {
	if (this.user != null && this.user.getCurrentSession() == this) {
	    this.user.setCurrentSession(null);
	}
        this.user = user;
	if (this.user != null) {
            this.user.setCurrentSession(this);
	}
    }

    /**
     * Sets the user reference to point to the user with the given Subject.
     *
     * @param subj the Subject for the user
     */
    public void setUser(Subject subj) throws Exception {
        setUser(LibertyUser.find(subj));
    }

    /**
     * Gets the list of service providers relying on the user's
     * current session.  Can be used when logging out the user.
     * @return the set containing the provider descriptors
     */
    public Set getRelyingSPList() {
        return spList;
    }

    /**
     * Adds an SP to the list that is relying on the current session.
     * @param prov the provider to add to the list
     */
    public void addRelyingSP(ProviderDescriptor prov) {
        if (spList == null) {
            spList = Collections.synchronizedSet(new HashSet());
        } 
        spList.add(prov);
    }

    /**
     * Returns the AuthnRequest that is currently pending.
     * To be used to validate certain parameters of the response when
     * it arrives.  Should be null if no request is pending.
     * @return an AuthnRequest, or null
     */
    public LibertyAuthnRequest getPendingAuthnRequest() {
	return pendingAuthnRequest;
    }

    /**
     * Sets or clears the pending AuthnRequest.  Used by the LoginServlet to
     * update this when a new AuthnRequest is generated.
     * @param value the new AuthnRequest, or null
     */
    public void setPendingAuthnRequest(LibertyAuthnRequest value) {
	pendingAuthnRequest = value;
    }

    /**
     * Returns the URL the UA should be redirected to after
     * processing by the AssertionServlet is complete.
     * @return the relay URL, or null if none
     */
    public String getPendingRelayURL() {
	return pendingRelayURL;
    }

    /**
     * Sets or clears the URL that the AssertionServlet should redirect
     * the UA to upon processing an incoming assertion.
     * @param value the new URL, or null
     */
    public void setPendingRelayURL(String value) {
	pendingRelayURL = value;
    }

    /**
     * Returns the provider ID of the IDP the pending AuthnRequest was
     * sent to.
     * @return the provider ID of the IDP
     */
    public String getPendingIDP() {
	return pendingIDP;
    }

    /**
     * Sets the provider ID of the IDP the pending AuthnRequest will be
     * sent to.
     * @param value the provider ID of the IDP
     */
    public void setPendingIDP(String value) {
	pendingIDP = value;
    }

    /**
     * Returns value of SLO (single log-out) flag
     * @return true if single log-out has already been initiated
     */
    public boolean getPendingSLO() {
        return pendingSLO;
    }

    /**
     * Sets the pending SLO (single log-out) flag.
     * Set to true when single log-out is initiated, set to false
     * when SLO is completed.  Used to determine appropriate action
     * when user agent accesses a logout servlet.
     */
    public void setPendingSLO(boolean value) {
        pendingSLO = value;
    }

    /**
     * Adds a URL to the list for the web bug logout page.
     * @param url the URL to add to the list.
     */
    public void addPendingSLOImage(String url) {
        pendingSLOImages.add(url);
    }

    /**
     * Gets the list of pending web bug URLs.
     * @return the list.
     */
    public List getPendingSLOImages() {
        return pendingSLOImages;
    }

    /**
     * Sets the web bug flag.  If true, the logout servlet will redirect
     * to the web bug logout page as part of the IDP logout process.
     * @param value The new value for the flag.
     */
    public void setPendingSLOWebBugs(boolean value) {
        pendingSLOWebBugs = value;
    }

    /**
     * Returns the value of the web bug flag.
     * @return true if the web bug logout step is needed
     */
    public boolean getPendingSLOWebBugs() {
        return pendingSLOWebBugs;
    }

    /**
     * Adds a URL to the list of pending URLs to redirect to for logout.
     * The IDP logout procedure will keep redirecting until the list is
     * empty.
     * @param url the URL to add to the list.
     */
    public void addPendingSLORedirect(String url) {
        pendingSLORedirects.add(url);
    }

    /**
     * Gets the list of pending redirect URLs for logout.  See above.
     * @return the list.
     */
    public List getPendingSLORedirects() {
        return pendingSLORedirects;
    }

    /**
     * Gets a reference URI for the authentication context profile for the
     * authentication method used to identify the user (getUser()) in this
     * session.
     *
     * @return	A string containing the AuthnContextProfileRef URI for the
     *		profile matching the user's authentication method.  Returns
     *		null if the user has not been authenticated.
     */
    public String getAuthnContextProfileRef() {
        return authnProfile;
    }

    /**
     * Stores a reference to an authentication context profile describing how
     * the user was authenticated.
     * @param authnProfile The URL of the profile, or null if none.
     */
    public void setAuthnContextProfileRef(String authnProfile) {
        this.authnProfile = authnProfile;
    }

    /**
     * Returns the assertion used to authenticate the user, or null if
     * the user was not authenticated using an assertion.
     * @return the assertion, or null.
     */
    public LibertyAssertion getAuthnAssertion() {
	return authnAssertion;
    }

    /**
     * Sets or clears the assertion used to authenticate the user.
     * @param value the new assertion, or null.
     */
    public void setAuthnAssertion(LibertyAssertion value) {
	authnAssertion = value;
    }

    /** 
     * Gets the current front-channel nonce.
     * @return the nonce.
     */
    public String getFCNonce() {
	return fcNonce;
    }

    /**
     * Generates a new nonce and returns it.  Invalidates the old nonce.
     * @return the new nonce.
     */
    public String newFCNonce() {
	byte[] bytes = new byte[12];
	try {
	    random.nextBytes(bytes);
	    fcNonce = URLUtils.base64Encode(bytes);
	} catch (Throwable e) {
	    fcNonce = null;
	    e.printStackTrace();
	}
	return fcNonce;
    }
}
