/*
 * $Id: ProviderDescriptorFactory.java,v 1.3.2.1 2002/09/17 01:52:26 jsergent Exp $
 * ProviderDescriptorFactory.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Interface to provider descriptor factory.  Allows lookup of provider
 * metadata.
 */
public abstract class ProviderDescriptorFactory {
    private static Map instances = new HashMap();
    
    /**
     * Returns an instance of the default implementation.
     * @return an instance of a subclass of ProviderDescriptorFactory
     */
    public static ProviderDescriptorFactory getInstance() {
        return getInstance(Config.providerDescriptorFactory);
    }
    
    /**
     * Returns an instance of the named implementation.
     * @return an instance of the class with the given name
     */
    public static ProviderDescriptorFactory getInstance(String impl) {
        ProviderDescriptorFactory f = (ProviderDescriptorFactory) instances.get(impl);
        if (f == null) {
	    if (Config.debug) { System.err.println("impl:" + impl); }
	    if (Config.debug) { System.err.println("f==null"); }
            synchronized (ProviderDescriptorFactory.class) {
                f = (ProviderDescriptorFactory) instances.get(impl);
                if (f == null) {
	            if (Config.debug) { System.err.println("f still ==null"); }
                    try {
                        Class cl = Class.forName(impl);
                        f = (ProviderDescriptorFactory) cl.newInstance();
                    } catch (Exception e) {
                        e.printStackTrace();
                        return null;
                    }
                    instances.put(impl, f);
                    if (Config.debug) { System.err.println("did put"); }
                    if (Config.debug) { System.err.println("get==" + instances.get(impl)); }
                }
            }
        }
        return f;
    }
    
    /**
     * Finds the SP with the given provider ID.
     * @param spProviderID the provider ID.
     * @return the SPDescriptor object
     */
    public SPDescriptor findSP(String spProviderID) {
        ProviderDescriptor prov = find(spProviderID);
        if (prov instanceof SPDescriptor) {
            return (SPDescriptor) prov;
        } else {
            return null;
        }
    }

    /**
     * Finds the SP with the given succinct ID.
     * @param spProviderSuccinctID the provider succinct ID.
     * @return the SPDescriptor object, or null if not found
     */
    public SPDescriptor findSP(byte[] spProviderSuccinctID) {
        ProviderDescriptor prov = find(spProviderSuccinctID);
        if (prov instanceof SPDescriptor) {
            return (SPDescriptor) prov;
        } else {
            return null;
        }
    }

    /**
     * Finds the IDP with the given provider ID.
     * @param idpProviderID the provider ID.
     * @return the IDPDescriptor object, or null if not found
     */
    public IDPDescriptor findIDP(String idpProviderID) {
        ProviderDescriptor prov = find(idpProviderID);
        if (prov instanceof IDPDescriptor) {
            return (IDPDescriptor) prov;
        } else {
            return null;
        }
    }

    /**
     * Finds the IDP with the given succinct ID.
     * @param idpProviderSuccinctID the provider succinct ID.
     * @return the IDPDescriptor object, or null if not found
     */
    public IDPDescriptor findIDP(byte[] idpProviderSuccinctID) {
        ProviderDescriptor prov = find(idpProviderSuccinctID);
        if (prov instanceof IDPDescriptor) {
            return (IDPDescriptor) prov;
        } else {
            return null;
        }
    }

    /**
     * Gets the provider descriptor of the local provider.
     * @return the ProviderDescriptor object (can be casted to SPDescriptor
     * or IDPDescriptor as appropriate)
     */
    public abstract ProviderDescriptor findLocal();

    /**
     * Finds the SP or IDP with the given provider ID.
     * @param providerID the provider ID
     * @return the ProviderDescriptor object (can be casted to SPDescriptor
     * or IDPDescriptor as appropriate)
     */
    public abstract ProviderDescriptor find(String providerID);

    /**
     * Finds the SP or IDP with the given provider succinct ID.
     * @param providerSuccinctID the provider succinct ID
     * @return the ProviderDescriptor object (can be casted to SPDescriptor
     * or IDPDescriptor as appropriate)
     */
    public abstract ProviderDescriptor find(byte[] providerSuccinctID);

    /**
     * Returns a list of SPDescriptor objects, one for each known 
     * SP.
     * @return the list.
     */
    public abstract List getSPList();

    /**
     * Returns a list of IDPDescriptor objects, one for each known 
     * IDP.
     * @return the list.
     */
    public abstract List getIDPList();
}
