/*
 * $Id: URLUtils.java,v 1.10.2.1 2002/09/17 01:52:27 jsergent Exp $
 * URLUtils.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.net.URLDecoder;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.util.Collection;
import java.util.Iterator;
import java.util.StringTokenizer;

/**
 * Utilities for URL manipulation.
 * @author Jonathan Sergent
 * @version $Revision: 1.10.2.1 $
 */
public class URLUtils {
    /**
     * Appends a new field to an x-www-form-urlencoded format string buffer.
     * The name and the value should not be pre-encoded.
     *
     * @param s the buffer
     * @param name the name of the new field
     * @param value the value of the new field
     */
    static void addQueryParam(StringBuffer s, String name, String value) {
	if (value == null || value.equals("")) {
	    return;
	}
	if (s.length() != 0) {
	    s.append('&');
	}
	s.append(URLEncoder.encode(name));
	s.append('=');
	s.append(URLEncoder.encode(value));
    }

    /**
     * Appends a new field to an x-www-form-urlencoded format string buffer.
     * The name and the value should not be pre-encoded.
     *
     * @param s the buffer
     * @param name the name of the new field
     * @param value the value of the new field
     */
    static void addQueryParam(StringBuffer s, String name, int value) {
	addQueryParam(s, name, String.valueOf(value));
    }

    /**
     * Appends a new field to an x-www-form-urlencoded format string buffer.
     * The name and the value should not be pre-encoded.
     *
     * @param s the buffer
     * @param name the name of the new field
     * @param value the value of the new field
     */
    static void addQueryParam(StringBuffer s, String name, boolean value) {
	addQueryParam(s, name, String.valueOf(value));
    }

    /**
     * Appends a new field to an x-www-form-urlencoded format string buffer.
     * The name and the value should not be pre-encoded.  The value is
     * formed by concatenating the members of the collection, seperated
     * by spaces.
     *
     * @param s the buffer
     * @param name the name of the new field
     * @param value the value of the new field
     */
    static void addQueryParam(StringBuffer s, String name, Collection coll) {
	if (coll != null) {
	    StringBuffer value = null;
	    Iterator i = coll.iterator();
	    while (i.hasNext()) {
		String ref = (String) i.next();
		if (value == null) {
		    value = new StringBuffer();
		}
		else {
		    value.append(' ');
		}
		value.append(ref);
	    }
	    addQueryParam(s, name, value.toString());
	}   
    }

    /**
     * Encodes the bytes using base64 and returns the encoded string.
     * @param data the data to be encoded
     * @return the base64-encoded data
     */
    public static String base64Encode(byte[] data) throws Exception {
	ByteArrayOutputStream os = new ByteArrayOutputStream();
	OutputStream enc = javax.mail.internet.MimeUtility.encode(os, "base64");
	enc.write(data);
	enc.close();
	return os.toString();
    }

    /**
     * Decodes base64 data into the original raw bytes.
     * @param data the string with the encoded data
     * @return the decoded raw bytes
     */
    public static byte[] base64Decode(String data) throws Exception {
	ByteArrayInputStream is = new ByteArrayInputStream(data.getBytes());
	InputStream dec = javax.mail.internet.MimeUtility.decode(is, "base64");
	byte[] a = new byte[dec.available()];
	int actual = dec.read(a);
	byte[] real = new byte[actual];
	System.arraycopy(a, 0, real, 0, actual);
	return real;
    }

    /**
     * Signs a query string as specified by Liberty.  Uses the key
     * configured in Config.
     * @param query the buffer to be signed.  The signature is appended.
     */
    static void sign(StringBuffer query) throws Exception {
	/*
	 LocalProvider pd;
	 pd = (LocalProvider) ProviderDirectory.instance()
	 .getLocalProviderDescriptor();
	 PrivateKey signingKey = pd.getPrivateKey();
	 */

	KeyStore ks = KeyStore.getInstance(KeyStore.getDefaultType());
	ks.load(new java.io.FileInputStream(Config.signingKeystore),
	        Config.signingKeystorePass.toCharArray());
	PrivateKey signingKey;
	signingKey = (PrivateKey) ks.getKey(Config.signingKeyAlias,
				            Config.signingKeyPass.toCharArray());
	query.append("&SigAlg=");
	query.append(URLEncoder.encode(Config.signingKeyAlg));
	String toSign = query.toString();
	String sig;
	try {
		Signature signature = Signature.getInstance("SHA1withRSA");
		signature.initSign(signingKey);
		signature.update(toSign.getBytes());
		sig = base64Encode(signature.sign());
	} catch (java.security.InvalidKeyException e) {
		Signature signature = Signature.getInstance("SHA1withDSA");
		signature.initSign(signingKey);
		signature.update(toSign.getBytes());
		sig = base64Encode(signature.sign());
	}
        query.append("&Signature=");
	query.append(URLEncoder.encode(sig));
    }

    /**
     * Verifies the signature on the given URL.
     * @param query the portion of the URL to be verified
     * @param sigdata the value of the Signature field in the URL
     * @param algorithm the signature algorithm
     * @param provider the provider whose key should be used for verification
     */
    static boolean verify(StringBuffer query, String sigdata, String algorithm, String provider) throws Exception {
	byte[] sigbytes = base64Decode(sigdata);
        ProviderDescriptor prov;
        prov = ProviderDescriptorFactory.getInstance().find(provider);
	if (prov == null) {
	    return false;
	}
	PublicKey key = prov.getKeyInfo();
	Signature signature = null;
	if (algorithm.endsWith("#rsa-sha1")) {
	    signature = Signature.getInstance("SHA1withRSA");
	} else if (algorithm.endsWith("#dsa-sha1")) {
	    signature = Signature.getInstance("SHA1withDSA");
	} else {
	    return false; /* Unsupported algorithm. */
	}
	signature.initVerify(key);
	signature.update(query.toString().getBytes());
	boolean rc = signature.verify(base64Decode(sigdata));
	return rc;
    }

    /**
     * This is for a weird hybrid of URL encoding and HTML post encoding --
     * generates HTML which causes an HTML browser to generate a form, which,
     * when posted, will send "query" as the query string.  Note that query
     * must be a well formed query string (key=value&key=value&...) for
     * this to work, and the browser may re-order the elements before
     * submitting them, so as things currently are this might break with
     * signed requests.
     * NOTE: This scheme is no longer part of the Liberty protocol spec.
     */
    public static String doHTMLConversion(String query, String destination) {
        StringTokenizer t = new StringTokenizer(query, "=&", false);
        /*
         * FIXME: Key and value must be HTML encoded!  Embedded special
         * characters must be encoded as entities (i.e. &quot; instead of ").
         */
        StringBuffer html = new StringBuffer();
        html.append("<form action=\"");
	html.append(destination);
	html.append("\" method=\"POST\">");

        while (t.hasMoreTokens()) {
            String key, value;
            try {
                key = t.nextToken();
                value = t.nextToken();
            }
            catch (java.util.NoSuchElementException e) {
                break;
            }
            html.append("<input type=\"hidden\" name=\"");
	    html.append(HTMLPostUtils.htmlEscape(URLDecoder.decode(key)));
	    html.append("\" value=\"");
	    html.append(HTMLPostUtils.htmlEscape(URLDecoder.decode(value)));
	    html.append("\"/>");
	}

        html.append("<input type=\"submit\"/></form>");
        return html.toString();
    }    

    public static Iterator tokenize(final String url) {
	return new Iterator() {
	    int lastPosition = -1;

	    public boolean hasNext() {
		return -1 != url.indexOf('=', lastPosition + 1);
	    }

	    public Object next() throws java.util.NoSuchElementException {
		int e = url.indexOf('=', lastPosition + 1);
		if (e == -1) { throw new java.util.NoSuchElementException(); }
		final String key = url.substring(lastPosition + 1, e);
		int a = url.indexOf('&', e + 1);
		String value;
		if (a == -1) {
		    value = url.substring(e + 1);
		    lastPosition = e;
		} else {
		    value = url.substring(e + 1, a);
		    lastPosition = a;
		}
		final String fvalue = value;
		if (Config.debug) {
		    System.err.println("next: key:" + key + " value:" + value);
		    System.err.println("e:" + e);
		    System.err.println("a:" + a);
		    System.err.println("lastPosition:" + lastPosition);
		}
		return new java.util.Map.Entry() {
		    public Object getKey() { return key; }
		    public Object getValue() { return fvalue; }
		    public Object setValue(Object value) throws UnsupportedOperationException {
			throw new UnsupportedOperationException();
		    }
		};
	    }

	    public void remove() throws UnsupportedOperationException {
		throw new UnsupportedOperationException();
	    }
	};
    }
}
