/*
 * $Id: XMLElement.java,v 1.13.2.1 2002/09/17 01:52:28 jsergent Exp $
 * XMLElement.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty;
import java.security.SecureRandom;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;

/**
 * Base class for XML data bindings.  Subclasses of XMLElement are data
 * bindings for the information contained in an individual XML element
 * and its children.
 * Provides common interface for manipulating such objects.
 * This provides some common utilities and constants for use when implementing
 * them. <p>
 * The utilities should probably be in XMLUtils, and not here.
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.13.2.1 $
 */
public abstract class XMLElement implements Comparable {
    /** XML Schema namespace URI */
    public static final String XMLNS_NS_URI = "http://www.w3.org/2000/xmlns/";
    /** XML Schema namespace URI */
    public static final String XSI_NS_URI = "http://www.w3.org/2001/XMLSchema-instance";
    /** XML Digital Signature namespace URI */
    public static final String DSIG_NS_URI = "http://www.w3.org/2000/09/xmldsig#";
    /** SOAP namespace URI */
    public static final String SOAP_NS_URI = "http://schemas.xmlsoap.org/soap/envelope/";

    /** Common random number instance */
    protected static SecureRandom secureRandom = new SecureRandom();

    /**
     * Formats the element represented by this object into XML.  Creates
     * a DOM Element object from the various settings of the object.
     * Does not insert the Element into the XML document; caller should
     * do this if desired.
     * @param doc The desired owner document for the element
     * @return The new Element
     */
    public abstract Element getElement(Document doc);

    public int compareTo(Object o) throws ClassCastException {
	return XMLUtils.compare(this, (XMLElement) o);
    }

    private static SimpleDateFormat dateFormat;
    static {
	dateFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
	dateFormat.setTimeZone(java.util.TimeZone.getTimeZone("GMT"));
    }

    /**
     * Utility function for formatting XML dates.
     * @param date the Date object
     * @return the text date string in XML format
     */
    protected static String formatDate(Date date) {
	return dateFormat.format(date);
    }

    /**
     * Utility function for parsing XML dates.
     * @param text the text date string in XML format
     * @return a Date object
     */
    protected static Date parseDate(String text) {
	try {
	    return dateFormat.parse(text);
	} catch (java.text.ParseException e) {
	    return null;
	}
    }

    /**
     * Utility function to format all of the XMLElements in a collection
     * and append them to a DOM Element.
     * @param e the Element to append to
     * @param c the collection of XMLElements to append
     */
    protected static void iterateAddChildren(Element e, Collection c) {
	if (c != null) {
	    Iterator i = c.iterator();
	    while (i.hasNext()) {
		XMLElement s = (XMLElement) i.next();
		e.appendChild(s.getElement(e.getOwnerDocument()));
	    }
	}
    }

    protected static void iterateAddNewChildrenNSWithText(Element e, String ns,
							  String name, Collection c)
    {
	if (c != null) {
	    Iterator i = c.iterator();
	    while (i.hasNext()) {
		String value = (String) i.next();
		appendNewChildNSWithText(e, ns, name, value);
	    }
	}
    }

    protected static void appendNewChildNSWithText(Element e, String ns, String name, String text) {
	if (text != null && !text.equals("")) {
	    Document doc = e.getOwnerDocument();
	    Element newElement = doc.createElementNS(ns, name);
	    e.appendChild(newElement);
	    Node textNode = doc.createTextNode(text);
	    newElement.appendChild(textNode);
	}
    }

    // this is probably the wrong way to do this
    // should instead have a way to override the name at getElement time
    protected static Element rename(Element e, String newName) {
	Document doc = e.getOwnerDocument();
	Element renamed = doc.createElement(newName);
	NamedNodeMap attrs = e.getAttributes();
	for (int i = 0; i < attrs.getLength(); i++) {
	    renamed.setAttributeNode((Attr) attrs.item(i));
	}
	for (Node i = e.getFirstChild(); i != null; i = i.getNextSibling()) {
	    renamed.appendChild(i.cloneNode(true));
	}
	return renamed;
    }

    /** Compatibility hack */
    public static boolean isElement(Element e, String ns, String name) {
        return XMLUtils.isElement(e, ns, name);
    }
}
